Method: NATS#connection_completed
- Defined in:
- lib/nats/client.rb
#connection_completed ⇒ Object
:nodoc:
1046 1047 1048 1049 1050 1051 1052 1053 1054 1055 1056 |
# File 'lib/nats/client.rb', line 1046 def connection_completed #:nodoc: @parse_state = AWAITING_INFO_LINE # Delay sending CONNECT or any other command here until we are sure # that we have a valid established secure connection. return if (@ssl or @tls) # Mark that we established already TCP connection to the server, # when using TLS we only do so after handshake has been completed. @connected = true end |